﻿#region "GNU General Public License"
/*
 *  Socket Server 2010 provides the ability to to have a socket server for testing
    Copyright (C) 2012  David Welford-Costelloe (http://www.welford-costelloe.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#endregion

using System;
using System.Diagnostics;
using System.Drawing;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using SocketServer2010.Classes;

namespace SocketServer2010
{
    public partial class MainFrm : Form
    {
        #region Variables

        private Utils oUtils = new Utils();
        private string sHostIPName = String.Empty;
        private int iPort = 0;
        private static TcpListener tcpListener;
        private static Thread listenThread;
        private static TcpClient tcpClient;
        static NetworkStream clientStream;

        /// <summary>
        /// Getter/Setter for iPort
        /// </summary>
        public int IPort
        {
            get { return iPort; }
            set { iPort = value; }
        }

        #endregion Variables

        #region Delegate Variables

        // The Windows From components are single thread
        // Using delegate will allows use to update the components
        private delegate void SetClientMessageResultsDelegate(string text);
        private delegate void SetLabelMessageDelegate(string sText);
        private delegate void SetStatusImageDelegate(Image img, string sToolTip);
        private delegate void SetButtonDelegate(Button btn, bool blnOption);
        private delegate void SetStatusTimeDelegate(string sText);

        #endregion Delegate Variables

        #region Application Start

        /// <summary>
        /// MainFrm Constructor
        /// </summary>
        public MainFrm()
        {
            InitializeComponent();
            initiateListen();
        }

        /// <summary>
        /// Main form for loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainFrm_Load(object sender, EventArgs e)
        {
            this.Icon = new Icon(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(AppConstants.MAIN_ICON_IMAGE));
            txtHostName.Text = oUtils.getIPAddress();
            sHostIPName = txtHostName.Text;
            setIconStatus();
            stripStatusDate.Text = DateTime.Now.ToShortDateString();
        }

        #endregion Application Start

        #region Initialization

        /// <summary>
        /// Start The listener using thread
        /// </summary>
        private void initiateListen()
        {
            this.stripStatusTime.Text = DateTime.Now.ToShortTimeString();
            // Set Server to listen on Port 4444
            IPort = 4444;
            tcpListener = new TcpListener(IPAddress.Any, IPort);
            listenThread = new Thread(new ThreadStart(ListenForClients));
            listenThread.Start();
            sendMsgToStatus("Starting Listener Thread on Port [" + IPort + "]");
        }

        #endregion Initialization

        #region Application Exit

        /// <summary>
        /// Application Exit confirm
        /// </summary>
        private void appExit()
        {
            sendMsgToStatus("User Requested Exit");
            DialogResult results = MessageBox.Show("Are you sure you wish to Exit Socket Client?",
               "Request to Close Application",
               MessageBoxButtons.YesNo,
               MessageBoxIcon.Question);

            if (results == DialogResult.Yes)
            {
                if (tcpListener != null)
                {
                    if (tcpClient != null)
                    {
                        tcpClient.Close();
                    }
                    tcpListener.Stop();
                    this.Dispose();
                }

                Application.Exit();
            }
            else
            {
                sendMsgToStatus("User Canceled Exit Application");
            }
        }

        #endregion Application Exit

        #region Client Listeners

        /// <summary>
        /// Listen For Client Connections
        /// </summary>
        private void ListenForClients()
        {
            try
            {
                sendMsgToStatus("Listener Started...waiting");
                tcpListener.Start();
            }
            catch (Exception e)
            {
                sendMessageBox("When Starting Listener an Exception was raised\n" + e.Message + "\nYou can ignore this message information only", "Information", MessageBoxIcon.Information);
            }

            while (true)
            {
                //blocks until a client has connected to the server
                try
                {
                    TcpClient client = tcpListener.AcceptTcpClient();

                    //create a thread to handle communication
                    //with connected client
                    Thread clientThread = new Thread(new ParameterizedThreadStart(HandleClientConnections));
                    clientThread.Start(client);
                }
                catch (SocketException)
                {
                    //sendMessageBox("Socket Exception thrown: " + sck, "Socket Exception", MessageBoxIcon.Warning);
                }
                catch (Exception)
                {
                }
            }
        }

        #endregion Client Listeners

        #region Listen Events

        /// <summary>
        /// Handles all client connections
        /// </summary>
        /// <param name="client">Client Object</param>
        private void HandleClientConnections(object client)
        {
            string sTextTemp = String.Empty;
            tcpClient = (TcpClient)client;
            NetworkStream clientStream = tcpClient.GetStream();
            int iBufferLength = 4096;

            // Populate the Byte array with the length of the message
            byte[] message = new byte[iBufferLength];
            int bytesRead;

            while (true)
            {
                bytesRead = 0;

                try
                {
                    //blocks until a client sends a message
                    bytesRead = clientStream.Read(message, 0, iBufferLength);
                }
                catch
                {
                    //a socket error has occurred
                    break;
                }

                if (bytesRead == 0)
                {
                    //the client has disconnected from the server
                    sTextTemp = "A Client has Disconnected from this server on Port: [" + IPort + "] at [" + DateTime.Now + "]";
                    sendMsgToStatus(sTextTemp);

                    break;
                }

                //message has successfully been received
                ASCIIEncoding encoder = new ASCIIEncoding();
                Debug.WriteLine(encoder.GetString(message, 0, bytesRead));
                sTextTemp = tcpClient.Client.LocalEndPoint + " " + tcpClient.Client.RemoteEndPoint + " " + encoder.GetString(message, 0, bytesRead);
                this.Invoke(new SetClientMessageResultsDelegate(SetResultsContents), sTextTemp);
            }

            tcpClient.Close();
        }

        #endregion Listen Events

        #region Button Events

        /// <summary>
        /// Setup and Listen for Clients
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnListener_Click(object sender, EventArgs e)
        {
            initiateListen();
            string sTemp = "Waiting for Clients to connect on Port " + IPort;
            sendMsgToStatus(sTemp);
            stripStatusText.Text = sTemp;
            setIconStatus();
        }

        /// <summary>
        /// Send Data to Client button event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSendToClient_Click(object sender, EventArgs e)
        {
            if (txtSendToClient.Text.Length > 0 && tcpListener != null)
            {
                clientStream = tcpClient.GetStream();
                ASCIIEncoding encoder = new ASCIIEncoding();
                byte[] buffer = encoder.GetBytes(txtSendToClient.Text);

                clientStream.Write(buffer, 0, buffer.Length);
                clientStream.Flush();

                sendMsgToStatus("Data has been sent to Client on Port [" + iPort + "]");
            }
            else
            {
                sendMsgToStatus("Please ensure you have entered data and are connected prior to sending...");
            }
        }

        /// <summary>
        /// Stop Listener on Socket Button Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStopListener_Click(object sender, EventArgs e)
        {
            if (tcpListener != null)
            {
                if (tcpListener.Server.Connected)
                {
                    tcpListener.Server.Close();
                }
            }

            setIconStatus();
        }

        /// <summary>
        /// Button Exit Click Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExit_Click(object sender, EventArgs e)
        {
            appExit();
        }

        #endregion Button Events

        #region Delegates

        /// <summary>
        /// Set the image on the StatusBar Using Invoke
        /// </summary>
        /// <param name="img"></param>
        /// <param name="sToolTip"></param>
        private void SetStatusImage(Image img, string sToolTip)
        {
            stripStatusImage.Image = img;
            stripStatusImage.ToolTipText = sToolTip;
        }

        /// <summary>
        /// Set the StatusBar Text Updates
        /// </summary>
        /// <param name="sText"></param>
        private void SetStatusMessage(string sText)
        {
            stripStatusText.Text = sText;
        }

        /// <summary>
        /// Due to Threading we must use the Invoke/Delagate
        /// for the Results TextBox
        /// </summary>
        /// <param name="sText">Message for Results TextBox</param>
        private void SetResultsContents(string sText)
        {
            this.txtResults.AppendText(sText);
        }

        /// <summary>
        /// Due to Threading we must use the Invoke/Delegate
        /// for the Results TextBox
        /// </summary>
        /// <param name="sText">Message for Results Label Control</param>
        private void SetLabelMessage(string sText)
        {
            this.lblText.Text = sText;
        }

        /// <summary>
        /// Enable or disable Buttons using Invoke
        /// </summary>
        /// <param name="btn"></param>
        /// <param name="blnOption"></param>
        private void SetButtonEnabled(Button btn, bool blnOption)
        {
            btn.Enabled = blnOption;
        }

        /// <summary>
        /// Set the Time on the Status Bar
        /// </summary>
        /// <param name="sText"></param>
        private void SetStatusDateTime(string sText)
        {
            stripStatusTime.Text = sText;
        }

        #endregion Delegates

        #region StatusBar

        /// <summary>
        /// Send a message to Statusbar
        /// </summary>
        /// <param name="sMSG"></param>
        private void sendMsgToStatus(string sMsg)
        {
            if (InvokeRequired)
            {
                this.Invoke(new SetLabelMessageDelegate(SetLabelMessage), sMsg);
            }
            else
            {
                lblText.Text = sMsg;
            }
        }

        #endregion StatusBar

        #region Timer Event

        /// <summary>
        /// Sets Icon Status and Clock
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            this.Invoke(new SetStatusTimeDelegate(SetStatusDateTime), DateTime.Now.ToShortTimeString());
            setIconStatus();
        }

        #endregion Timer Event

        #region Change Icon Images

        /// <summary>
        /// Sets online or Offline StatusBar image
        /// </summary>
        private void setIconStatus()
        {
            string sWait = stripStatusImage.ToolTipText = AppConstants.WAITLINE_TEXT + " on Port [" + IPort + "]";
            string sOn = stripStatusImage.ToolTipText = "Client Currently Connected to [" + sHostIPName + "]" + " On Port [" + iPort + "]";
            string sOf = stripStatusImage.ToolTipText = AppConstants.OFFLINE_TEXT;

            // We have to check for tcpListener
            if (checkListenerConnection())
            {
                Bitmap image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(AppConstants.ON_LINE_WAIT_IMAGE));
                this.Invoke(new SetStatusImageDelegate(SetStatusImage), image, sWait);
                sendMsgToStatus(sWait);
                stripStatusText.Text = sWait;

                if (btnSendToClient.InvokeRequired)
                {
                    this.Invoke(new SetButtonDelegate(SetButtonEnabled), this.btnSendToClient, false);
                }
                else
                {
                    btnSendToClient.Enabled = false;
                }
            }
            else
            {
                Bitmap image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(AppConstants.OFF_LINE_ICON_IMAGE));
                this.Invoke(new SetStatusImageDelegate(SetStatusImage), image, sOf);
                sendMsgToStatus(sOf);
                stripStatusText.Text = sOf;
            }

            // Check Client Connection
            if (checkClientConnection())
            {
                Bitmap image = new Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(AppConstants.ON_LINE_ICON_IMAGE));
                this.Invoke(new SetStatusImageDelegate(SetStatusImage), image, sOn);
                sendMsgToStatus(sOn);
                stripStatusText.Text = sOn;
            }
        }

        #endregion Change Icon Images

        #region Menu Events

        /// <summary>
        /// Application Exit Method
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            appExit();
        }

        /// <summary>
        /// Menu Item Browser Click Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dWCSiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Browser frm = new Browser();
            frm.ShowDialog();
        }

        /// <summary>
        /// Menu Item AboutBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox frm = new AboutBox();
            frm.ShowDialog();
            frm.Dispose();
        }

        #endregion Menu Events

        #region Validating Events

        /// <summary>
        /// Validating event for Send to Client Data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtSendToClient_TextChanged(object sender, EventArgs e)
        {
            if (txtSendToClient.Text.Length > 0)
            {
                this.Invoke(new SetButtonDelegate(SetButtonEnabled), this.btnClearSend, true);
                this.Invoke(new SetButtonDelegate(SetButtonEnabled), this.btnSendToClient, true);
            }
            else
            {
                this.Invoke(new SetButtonDelegate(SetButtonEnabled), this.btnClearSend, false);
                this.Invoke(new SetButtonDelegate(SetButtonEnabled), this.btnSendToClient, false);
            }
        }

        #endregion Validating Events

        #region Validate TCPSockets

        /// <summary>
        /// Check if Listener is connected waiting for Clients
        /// </summary>
        /// <returns></returns>
        private bool checkListenerConnection()
        {
            bool blnCon = false;

            if (tcpListener != null)
            {
                blnCon = true;
            }
            else
            {
                blnCon = false;
            }
            return blnCon;
        }

        /// <summary>
        /// Check to to if a Client is connected
        /// </summary>
        /// <returns></returns>
        private bool checkClientConnection()
        {
            bool blnCon = false;

            if (tcpClient != null)
            {
                blnCon = tcpClient.Connected;
            }
            else
            {
                blnCon = false;
            }

            return blnCon;
        }

        #endregion Validate TCPSockets

        #region Send MessageBox

        /// <summary>
        /// Send Message Box to User
        /// </summary>
        /// <param name="sMsg"></param>
        /// <param name="sTitle"></param>
        /// <param name="oIcon"></param>
        private void sendMessageBox(string sMsg, string sTitle, MessageBoxIcon oIcon)
        {
            MessageBox.Show(sMsg,
            sTitle,
            MessageBoxButtons.OK,
            oIcon,
            MessageBoxDefaultButton.Button1);
        }

        #endregion Send MessageBox
    }
}